@@ -1,6 +1,6 @@
 import sys
 import logging
-import gobject
+import glib
 import gtk
 import deskbar
 import deskbar.interfaces.Match
@@ -297,8 +297,8 @@ class CoreImpl(deskbar.interfaces.Core):
         """
         if (len(text) >= self.get_min_chars()):            
             if (self._start_query_id != 0):
-                gobject.source_remove(self._start_query_id)
-            self._start_query_id = gobject.timeout_add( self.get_type_delay(), self._query_real, text )
+                glib.source_remove(self._start_query_id)
+            self._start_query_id = glib.timeout_add( self.get_type_delay(), self._query_real, text )
             self._stop_queries = False
             
     def _query_real(self, text):
@@ -380,7 +380,7 @@ class CoreImpl(deskbar.interfaces.Core):
         self.update_modules_priority(enabled_modules)
             
     def forward_query_ready(self, handler, query, matches):
-        if query == self._last_query and matches != None and len(matches) > 0 and not self._stop_queries:
+        if query == self._last_query and matches != None and not self._stop_queries:
             for match in matches:
                 if not isinstance(match, deskbar.interfaces.Match):
                     raise TypeError("Handler %r returned an invalid match: %r", handler,  match)
