@@ -7,11 +7,11 @@ class SendEmailToAction(ShowUrlAction):
     """
     
     def __init__(self, name, email):
-        ShowUrlAction.__init__(self, name, "mailto:"+email)
+        ShowUrlAction.__init__(self, name, "mailto: \"%s\" <%s>" % (name, email))
         self._email = email
     
     def get_icon(self):
-        return "stock_mail-send"
+        return "stock_mail-compose"
     
     def get_name(self, text=None):
         return {
