@@ -1,6 +1,7 @@
 import deskbar.interfaces.Action
 from gettext import gettext as _
-from deskbar.core.Utils import url_show
+from deskbar.core.Utils import url_show, uri_has_default_handler
+import cgi
 
 class ShowUrlAction(deskbar.interfaces.Action):
     """
@@ -27,4 +28,7 @@ class ShowUrlAction(deskbar.interfaces.Action):
         url_show(self._url)
         
     def get_tooltip(self, text=None):
-        return self._url
\ No newline at end of file
+        return cgi.escape(self._url)
+    
+    def is_valid(self):
+        return uri_has_default_handler(self._url)
\ No newline at end of file
