@@ -1,5 +1,5 @@
 from deskbar.core.GconfStore import GconfStore
-from deskbar.core.Utils import strip_html, get_proxy, get_locale_lang
+from deskbar.core.Utils import strip_html, get_proxy, get_locale_lang, htmldecode
 from deskbar.defs import VERSION
 from deskbar.handlers.actions.CopyToClipboardAction import CopyToClipboardAction
 from deskbar.handlers.actions.ShowUrlAction import ShowUrlAction
@@ -30,8 +30,8 @@ LANGUAGES = (
     (_("Arabic"), "lang_ar"),
     (_("Bulgarian"), "lang_bg"),
     (_("Catalan"), "lang_ca"),
-    (_("Chinese simplified"), "lang_zh-CN"),
-    (_("Chinese traditional"), "lang_zh-TW"),
+    (_("Chinese (Simplified)"), "lang_zh-CN"),
+    (_("Chinese (Traditional)"), "lang_zh-TW"),
     (_("Croatian"), "lang_hr"),
     (_("Czech"), "lang_cs"),
     (_("Danish"), "lang_da"),
@@ -150,7 +150,7 @@ class GoogleHandler(deskbar.interfaces.Module):
         
         try:
             stream = urllib.urlopen(url, proxies=get_proxy())
-        except IOError, msg:
+        except (IOError, EOFError), msg:
             LOGGER.error("Could not open URL %s: %s, %s", url, msg[0], msg[1])
             return
             
@@ -163,9 +163,14 @@ class GoogleHandler(deskbar.interfaces.Module):
         LOGGER.debug("Returning Google answer for: %s", qstring)
         
         if not 'Error' in results:
-            for result in results:
-                matches.append( GoogleMatch(result) )
-            self._emit_query_ready(qstring, matches)
+            num_results = len(results)
+            for i, result in enumerate(results):
+                print result
+                prio = self.get_priority() + num_results - i
+                matches.append(GoogleMatch(result, priority=prio))
+            
+        matches.append(GoogleSearchForMatch(qstring, priority=self.get_priority()))    
+        self._emit_query_ready(qstring, matches)
 
     def has_config(self):
         return True
@@ -203,9 +208,9 @@ class OpenGoogleAction(ShowUrlAction):
 class GoogleMatch(deskbar.interfaces.Match):
     def __init__(self, result=None, **args):
         deskbar.interfaces.Match.__init__(self, category="web", icon="google.png", **args)
-        self._name = result['titleNoFormatting']
+        self._name = htmldecode(result['titleNoFormatting'])
         self.url = result['url']
-        self.set_snippet(result['content'])
+        self.set_snippet(htmldecode(result['content']))
         
         self.add_action( OpenGoogleAction(self.get_name(), self.url) )
         self.add_action( CopyToClipboardAction( _("URL"), self.url) )
@@ -214,6 +219,34 @@ class GoogleMatch(deskbar.interfaces.Match):
         return self.url
         
 
+class SearchWithGoogleAction(ShowUrlAction):
+    """
+    Open the Google search page with results
+    for the given query
+    """
+    
+    BASE_URL = "http://www.google.com/search?%s"
+    
+    def __init__(self, term):
+        url = self.BASE_URL % urllib.urlencode({'q': term})
+        ShowUrlAction.__init__(self, term, url)
+
+    def get_verb(self):
+        return _("Search <b>Google</b> for <i>%(name)s</i>")
+
+class GoogleSearchForMatch(deskbar.interfaces.Match):
+    """
+    Search Google for the given query
+    """
+    
+    def __init__(self, term=None, **args):
+        deskbar.interfaces.Match.__init__ (self, category="web", icon="google.png", **args)
+        self._term = term
+        self.add_action( SearchWithGoogleAction(self._term) )
+    
+    def get_hash(self):
+        return "google:"+self._term
+
 class GoogleConfigDialog(gtk.Dialog):
     """
     
